import React, { useState, useEffect } from 'react';
import { BrowserRouter as Router, Routes, Route, Navigate } from 'react-router-dom';
// import { Container } from 'react-bootstrap';
import { ErrorBoundary } from 'react-error-boundary';
import NavBar from './components/NavBar';
import Home from './pages/Home';
import About from './pages/About';
import Login from './pages/Login';
import Logout from './pages/Logout';
import Profile from './pages/Profile';
import Register from './pages/Register';
import CartPage from './pages/CartPage';
import TokenExpired from './pages/TokenExpired';
import TokenChecker from './components/TokenChecker';
import Banner from './components/Banner';
import AnnouncementBar from './components/Announcement-bar';
import UpdateProfile from './components/UpdateProfile';
import AdminDashboard from './pages/AdminDashboard';
import ProductPage from './pages/ProductPage';
import CheckoutPage from './pages/CheckoutPage';
import Error from './pages/Error'; // Import the NotFound component
import { ToastContainer } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';
import { UserProvider } from './UserContext';

function ErrorFallback({ error }) {
  return (
    <div>
      <h1>Something went wrong!</h1>
      <p>{error.message}</p>
    </div>
  );
}

function App() {
  const [user, setUser] = useState({
    id: null,
    isAdmin: null,
    isVerified: true,
  });

  const unsetUser = () => {
    localStorage.clear();
  };

  useEffect(() => {
    const checkTokenExpiration = async () => {
      const token = localStorage.getItem('token');
    
      if (token) {
        try {
          const response = await fetch(`${process.env.REACT_APP_API_URL}/user/details`, {
            headers: {
              Authorization: `Bearer ${token}`,
            },
          });
    
          if (response.ok) {
            const data = await response.json();
    
            if (typeof data._id !== 'undefined') {
              setUser({
                id: data._id,
                isAdmin: data.isAdmin,
              });
            } else {
              setUser({
                id: null,
                isAdmin: null,
              });
              unsetUser();
            }
          } else {
            // Handle non-OK responses
            console.error('Non-OK response:', response.status, response.statusText);
          }
        } catch (error) {
          // Handle fetch errors
          console.error('Fetch error:', error);
        }
      }
    };
    

    checkTokenExpiration();
  }, []);

  return (
    <UserProvider value={{ user, setUser, unsetUser }}>
      <Router>
        <TokenChecker />
        <ErrorBoundary FallbackComponent={ErrorFallback}>
          <AnnouncementBar />
          <NavBar />
          
          <Routes>
            <Route path="/" element={<Home />} />
            <Route path="/token-expired" element={<TokenExpired />} />
            <Route path="/register" element={<Register />} />
            <Route path="/login" element={<Login />} />
            <Route path="/dashboard" element={<AdminDashboard />} />
            <Route path="/profile" element={<Profile />} />
            <Route path="/logout" element={<Logout />} />
            <Route path="/about" element={<About />} />
            <Route path="/banner" element={<Banner />} />
            <Route path="/cart" element={<CartPage />} />
            <Route path="/update-profile" element={<UpdateProfile />} />
            <Route path="/404" element={<Error />} />
            <Route path="/product/:productId" element={<ProductPage />} />
            <Route path="/checkout" element={<CheckoutPage />} />
            {/* Catch-all route that redirects to the home page */}
            <Route path="*" element={<Navigate to="/404" />} />
          </Routes>
          <ToastContainer />
          
        </ErrorBoundary>
      </Router>
    </UserProvider>
  );
}

export default App;
