import React, { useState } from 'react';
import { Form, Button } from 'react-bootstrap';
import { toast } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';

const CreateProduct = () => {
  const [name, setName] = useState('');
  const [description, setDescription] = useState('');
  const [price, setPrice] = useState('');
  const [image, setImage] = useState(null);
  const [isActive, setIsActive] = useState(true);
  const [tags, setTags] = useState([]); // Changed to array for tags
  const [isSubmitting, setIsSubmitting] = useState(false);

  const handleImageChange = (e) => {
    const selectedImage = e.target.files[0];
    setImage(selectedImage);
  };

  const handleSubmit = async (e) => {
    e.preventDefault();

    try {
      setIsSubmitting(true);

      const formData = new FormData();
      formData.append('name', name);
      formData.append('description', description);
      formData.append('price', price);
      formData.append('tags', JSON.stringify(tags)); // Convert tags array to JSON string
      formData.append('isActive', isActive);
      formData.append('image', image);

      const token = localStorage.getItem('token');

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/product/create`,
        {
          method: 'POST',
          headers: {
            Authorization: `Bearer ${token}`,
          },
          body: formData,
        }
      );

      if (response.ok) {
        toast.success('Product created successfully', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });

        // Redirect to AllProducts page
      } else {
        toast.error('Failed to create product. Please check your input and try again.', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error('Error creating product:', error);

      toast.error('An unexpected error occurred. Please try again later.', {
        position: 'bottom-right',
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    } finally {
      setIsSubmitting(false);
    }
  };

  return (
    <Form onSubmit={handleSubmit}>
      <h1 className="my-5 text-center">Create Product</h1>

      <Form.Group controlId="productName">
        <Form.Label>Name</Form.Label>
        <Form.Control
          type="text"
          placeholder="Enter product name"
          value={name}
          onChange={(e) => setName(e.target.value)}
          required
        />
      </Form.Group>

      <Form.Group controlId="productDescription">
        <Form.Label>Description</Form.Label>
        <Form.Control
          as="textarea"
          placeholder="Enter product description"
          value={description}
          onChange={(e) => setDescription(e.target.value)}
          required
        />
      </Form.Group>

      <Form.Group controlId="productPrice">
        <Form.Label>Price</Form.Label>
        <Form.Control
          type="number"
          placeholder="Enter product price"
          value={price}
          onChange={(e) => setPrice(e.target.value)}
          required
        />
      </Form.Group>

      <Form.Group controlId="productTag">
        <Form.Label>Tag</Form.Label>
        <Form.Control
          type="text"
          placeholder="Enter product tag"
          value={tags.join(', ')} // Display tags as a comma-separated string
          onChange={(e) => setTags(e.target.value.split(',').map(tag => tag.trim()))} // Convert string to array
        />
      </Form.Group>

      <Form.Group controlId="productStatus">
        <Form.Check
          type="checkbox"
          label="Set Product to Active"
          checked={isActive}
          onChange={() => setIsActive(!isActive)}
        />
      </Form.Group>

      <Form.Group controlId="productImage">
        <Form.Label>Image</Form.Label>
        <Form.Control
          type="file"
          accept="image/*"
          name="image"
          onChange={handleImageChange}
          required
        />
      </Form.Group>

      <Button
        variant="primary"
        type="submit"
        disabled={isSubmitting}
      >
        {isSubmitting ? 'Creating...' : 'Create Product'}
      </Button>
    </Form>
  );
};

export default CreateProduct;
