// UpdateProfile.js

import React, { useContext, useEffect, useState } from 'react';
import { Form, Button, Alert, Container, Row, Col } from 'react-bootstrap';
import { toast } from 'react-toastify';
import UserContext from '../UserContext';

export default function UpdateProfile() {
  const { user } = useContext(UserContext);
  const [formData, setFormData] = useState({
    userId: user.id,
    newEmail: user.email,
    newFirstName: user.firstName,
    newLastName: user.lastName,
    newPassword: '',
    newMobileNo: user.mobileNo || '',
  });
  const [loading, setLoading] = useState(false);
  const [updateError, setUpdateError] = useState(null);

  const handleUpdateProfile = async (e) => {
    e.preventDefault();

    try {
      setLoading(true);

      const headers = {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${localStorage.getItem('token')}`,
      };

      const response = await fetch(`${process.env.REACT_APP_API_URL}/user/update`, {
        method: 'PUT',
        headers,
        body: JSON.stringify(formData),
      });

      if (response.ok) {
        const updatedUserData = await response.json();
        console.log('Updated User Data:', updatedUserData);

        // Handle the updated user data as needed
        toast.success('Profile updated successfully!', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      } else {
        console.error('Failed to update user profile:', response.status);
        const errorData = await response.json();
        setUpdateError(errorData.message);
        toast.error('Failed to update profile. Please try again.', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
          progress: undefined,
        });
      }
    } catch (error) {
      console.error('Error updating user profile:', error);
      setUpdateError('An unexpected error occurred. Please try again later.');
      toast.error('An unexpected error occurred. Please try again later.', {
        position: 'bottom-right',
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    // Update form data when user data changes
    setFormData({
      userId: user.id,
      newEmail: user.email,
      newFirstName: user.firstName,
      newLastName: user.lastName,
      newPassword: '',
      newMobileNo: user.mobileNo || '',
    });
  }, [user]);

  return (
    <Container className="mt-5 shadow p-5 rounded">
      <Row className="justify-content-center">
        <Col lg={6} offset={3}>
          <Form onSubmit={handleUpdateProfile}>
            <h1 className="my-5 text-center">Update Profile</h1>

            {updateError && <Alert variant="danger">{updateError}</Alert>}

            <Form.Group controlId="newEmail">
              <Form.Label>Email address</Form.Label>
              <Form.Control
                type="email"
                placeholder="Enter email"
                value={formData.newEmail}
                onChange={(e) => setFormData({ ...formData, newEmail: e.target.value })}
              />
            </Form.Group>

            <Form.Group controlId="newFirstName">
              <Form.Label className="mt-2">First Name</Form.Label>
              <Form.Control
                type="text"
                placeholder="Enter first name"
                value={formData.newFirstName}
                onChange={(e) => setFormData({ ...formData, newFirstName: e.target.value })}
              />
            </Form.Group>

            <Form.Group controlId="newLastName">
              <Form.Label className="mt-2">Last Name</Form.Label>
              <Form.Control
                type="text"
                placeholder="Enter last name"
                value={formData.newLastName}
                onChange={(e) => setFormData({ ...formData, newLastName: e.target.value })}
              />
            </Form.Group>

            <Form.Group controlId="newMobileNo">
              <Form.Label className="mt-2">Mobile Number</Form.Label>
              <Form.Control
                type="text"
                placeholder="Enter mobile number"
                value={formData.newMobileNo}
                onChange={(e) => setFormData({ ...formData, newMobileNo: e.target.value })}
              />
            </Form.Group>

            <Form.Group controlId="newPassword">
              <Form.Label className="mt-2">New Password</Form.Label>
              <Form.Control
                type="password"
                placeholder="Enter new password"
                value={formData.newPassword}
                onChange={(e) => setFormData({ ...formData, newPassword: e.target.value })}
              />
            </Form.Group>

            {loading ? (
              <Button className="mt-3" variant="primary" type="submit" disabled>
                Updating...
              </Button>
            ) : (
              <Button className="mt-3" variant="primary" type="submit">
                Save Changes
              </Button>
            )}
          </Form>
        </Col>
      </Row>
    </Container>
  );
}
