import React, { useState, useEffect } from "react";
import { Container, Row, Col, Table, Button, Form } from "react-bootstrap";
import { jwtDecode } from "jwt-decode";
import { ToastContainer, toast } from "react-toastify";
import { useNavigate } from "react-router-dom";
import "react-toastify/dist/ReactToastify.css";

const CartPage = () => {
  const [cartItems, setCartItems] = useState([]);
  const [productDetails, setProductDetails] = useState({});
  const [totalPrice, setTotalPrice] = useState(0);
  const [shippingFee, setShippingFee] = useState(0);
  const [promoCode, setPromoCode] = useState("");
  const navigate = useNavigate();

  const fetchCartItems = async () => {
    try {
      const token = localStorage.getItem("token");
      const decoded = jwtDecode(token);

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/cart/user/${decoded.userId}`,
        {
          method: "GET",
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );

      const data = await response.json();

      if (Array.isArray(data.items)) {
        setCartItems(data.items);
      }
    } catch (error) {
      console.error("Error fetching cart items:", error);
    }
  };

  useEffect(() => {
    fetchCartItems();
  }, []);

  useEffect(() => {
    const fetchProductDetails = async () => {
      try {
        const token = localStorage.getItem("token");
        const productIds = cartItems.map((item) => item.product);

        const promises = productIds.map(async (productId) => {
          const response = await fetch(
            `${process.env.REACT_APP_API_URL}/product/${productId}`,
            {
              method: "GET",
              headers: {
                Authorization: `Bearer ${token}`,
              },
            }
          );

          if (!response.ok) {
            console.error(
              `Failed to fetch product details for ID ${productId}: ${response.status}`
            );
            return null;
          }

          const productData = await response.json();
          return { [productId]: productData };
        });

        const productDetailsArray = await Promise.all(promises);
        const productDetailsObject = productDetailsArray.reduce(
          (acc, product) => ({ ...acc, ...product }),
          {}
        );
        setProductDetails(productDetailsObject);
      } catch (error) {
        console.error("Error fetching product details:", error);
      }
    };

    if (cartItems.length > 0) {
      fetchProductDetails();
    }
  }, [cartItems]);

  useEffect(() => {
    const calculateTotalPrice = () => {
      const totalPrice = cartItems.reduce((total, item) => {
        const product = productDetails[item.product];
        if (product) {
          return total + product.price * item.quantity;
        }
        return total;
      }, 0);

      setTotalPrice(totalPrice);
    };

    const calculateShippingFee = () => {
      // Leave it blank for now, add your logic to calculate shipping fee here
    };

    calculateTotalPrice();
    calculateShippingFee();
  }, [cartItems, productDetails]);

  const updateCart = async () => {
    try {
      const token = localStorage.getItem("token");
      const decoded = jwtDecode(token);

      const updates = cartItems.map((item) => ({
        product: item.product,
        quantity: item.quantity,
      }));

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/cart/update-cart`,
        {
          method: "PUT",
          headers: {
            Authorization: `Bearer ${token}`,
            "Content-Type": "application/json",
          },
          body: JSON.stringify({
            userId: decoded.userId,
            updates,
          }),
        }
      );

      const data = await response.json();

      if (response.ok) {
        toast.success(data.message, {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });

        fetchCartItems();
      } else {
        toast.error(data.error || "Error updating cart", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error updating cart:", error);
    }
  };

  const handleQuantityChange = (productId, newQuantity) => {
    setCartItems((prevItems) =>
      prevItems.map((item) =>
        item.product === productId
          ? { ...item, quantity: parseInt(newQuantity, 10) }
          : item
      )
    );
  };

  const handleProceedToCheckout = () => {
    // Add logic to handle the checkout process
    // You can navigate to the checkout page or display a modal for further steps
    navigate("/checkout");
    // This is a placeholder, you need to implement the actual checkout logic
    toast.success("Proceeding to Checkout...", {
      position: "bottom-right",
      autoClose: 2000,
      hideProgressBar: true,
      closeOnClick: true,
      pauseOnHover: true,
      draggable: true,
    });
  };

  return (
    <Container className="my-5 text-center">
      <h2>Shopping Cart</h2>
      {cartItems.length === 0 ? (
        <p>Your cart is empty.</p>
      ) : (
        <>
          <Row>
            {/* Left side - Product Details */}
            <Col xs={12} md={8}>
              <h3>Product Details</h3>
              {cartItems.map((item) => (
                <Table
                  key={item.product}
                  striped
                  bordered
                  hover
                  className="mb-4"
                >
                  <tbody>
                    <tr>
                      <td colSpan="2">
                        <strong>Product Image</strong>
                      </td>
                    </tr>
                    <tr>
                      <td colSpan="2">
                        <img
                          src={
                            (productDetails[item.product] &&
                              productDetails[item.product].image) ||
                            "placeholder-image-url"
                          }
                          alt={
                            (productDetails[item.product] &&
                              productDetails[item.product].name) ||
                            "Product Image"
                          }
                          style={{ width: "200px", height: "200px" }}
                        />
                      </td>
                    </tr>
                    <tr>
                      <td>
                        <strong>Name</strong>
                      </td>
                      <td>{productDetails[item.product]?.name}</td>
                    </tr>
                    <tr>
                      <td>
                        <strong>Description</strong>
                      </td>
                      <td>{productDetails[item.product]?.description}</td>
                    </tr>
                    <tr>
                      <td>
                        <strong>Price</strong>
                      </td>
                      <td>
                        <strong>
                          ₱{productDetails[item.product]?.price * item.quantity}
                        </strong>
                      </td>
                    </tr>
                    <tr>
                      <td>
                        <strong>Quantity</strong>
                      </td>
                      <td>
                        <input
                          type="number"
                          min="1"
                          value={item.quantity}
                          onChange={(e) =>
                            handleQuantityChange(item.product, e.target.value)
                          }
                        />
                      </td>
                    </tr>
                  </tbody>
                </Table>
              ))}
            </Col>

            {/* Right side - Order Summary */}
            <Col xs={12} md={4}>
              <h3>Order Summary</h3>
              <Table striped bordered hover>
                <thead>
                  <tr>
                    <th>Order Summary</th>
                    <th></th>
                  </tr>
                </thead>
                <tbody>
                  <tr>
                    <td>Total Items</td>
                    {cartItems.reduce(
                      (total, item) => total + item.quantity,
                      0
                    )}
                  </tr>
                  <tr>
                    <td>Shipping Fee</td>
                    <td>{shippingFee}</td>
                  </tr>
                  <tr>
                    <td>Total</td>
                    <td>
                      <strong>₱{totalPrice + shippingFee}</strong>
                    </td>
                  </tr>
                  <tr>
                    <td colSpan="2">
                      <Form.Group controlId="promoCode">
                        <Form.Label>Promo Code</Form.Label>
                        <Form.Control
                          type="text"
                          placeholder="Enter promo code"
                          value={promoCode}
                          onChange={(e) => setPromoCode(e.target.value)}
                          style={{ backgroundColor: "#f8f9fa" }}
                        />
                      </Form.Group>
                    </td>
                  </tr>
                </tbody>
              </Table>
              <div>
                <Button variant="primary" onClick={updateCart} className="mt-3">
                  Update Cart
                </Button>
                <Button
                  variant="primary"
                  onClick={handleProceedToCheckout}
                  className="mx-3 mt-3"
                >
                  Proceed to Checkout
                </Button>
              </div>
              <tr>
                <td
                  colSpan="2"
                  className="text-start"
                  style={{
                    fontSize: "12px",
                    opacity: "0.5",
                    lineHeight: "5px",
                  }}
                >
                  <p className="mt-4">
                    We accept cash, COD, bank deposit, credit card, Dragonpay,
                    and Paypal.
                  </p>
                  <p>(Cash on Delivery available within Metro Manila only)</p>
                  <p>Shipped through trusted couriers</p>
                  <p>7 days store replacement w/ warranty</p>
                </td>
              </tr>
            </Col>
          </Row>
        </>
      )}
      <ToastContainer />
    </Container>
  );
};

export default CartPage;
