// Login.js

import React, { useState, useEffect, useContext } from 'react';
import { Form, Button, Alert, Container, Col } from 'react-bootstrap';
import { toast } from 'react-toastify';
import { Navigate, useNavigate } from 'react-router-dom';
import UserContext from '../UserContext';

export default function Login() {
  const { user, setUser } = useContext(UserContext);
  const [email, setEmail] = useState('');
  const [password, setPassword] = useState('');
  const [isActive, setIsActive] = useState(false);
  const [loginError, setLoginError] = useState(null);
  const navigate = useNavigate();

  const authenticate = async (e) => {
    e.preventDefault();

    try {
      const response = await fetch(`${process.env.REACT_APP_API_URL}/user/authenticate`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          email: email,
          password: password,
        }),
      });

      const data = await response.json();

      if (response.ok && data.token) {
        localStorage.setItem('token', data.token);
        retrieveUserDetails(data.token);

        setLoginError(null);
        setIsActive(false);

        toast.success('Login successful!', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });

        if (data.isAdmin) {
          navigate('/dashboard');
        } else {
          navigate('/');
        }
      } else {
        setLoginError('Authentication failed. Check your login details and try again.');

        toast.error('Authentication failed. Check your login details and try again.', {
          position: 'bottom-right',
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error('Error during login:', error);

      setLoginError('An unexpected error occurred. Please try again later.');

      toast.error('An unexpected error occurred. Please try again later.', {
        position: 'bottom-right',
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }

    setEmail('');
    setPassword('');
  };

  const retrieveUserDetails = async (token) => {
    try {
      const response = await fetch(`${process.env.REACT_APP_API_URL}/user/details`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });

      const data = await response.json();

      if (response.ok) {
        setUser({
          id: data._id,
          isAdmin: data.isAdmin,
        });
      } else {
        console.error('Error retrieving user details:', data);
      }
    } catch (error) {
      console.error('Error during user details retrieval:', error);
    }
  };

  useEffect(() => {
    setIsActive(email !== '' && password !== '');
  }, [email, password]);

  return (
    user.id !== null ? (
      <Navigate to="/" />
    ) : (
      <Container className="my-5">
        <Col md={{ span: 6, offset: 3 }}>
          <Form onSubmit={(e) => authenticate(e)} className="shadow p-5 mb-5 bg-white rounded">
            <h1 className="text-center mb-4">Login</h1>

            {loginError && <Alert variant="danger">{loginError}</Alert>}

            <Form.Group controlId="userEmail">
              <Form.Label>Email address</Form.Label>
              <Form.Control
                type="email"
                placeholder="Enter email"
                value={email}
                onChange={(e) => setEmail(e.target.value)}
                required
              />
            </Form.Group>

            <Form.Group controlId="password">
              <Form.Label>Password</Form.Label>
              <Form.Control
                type="password"
                placeholder="Password"
                value={password}
                onChange={(e) => setPassword(e.target.value)}
                required
              />
            </Form.Group>

            {isActive ? (
              <Button variant="primary" type="submit" id="submitBtn">
                Submit
              </Button>
            ) : (
              <Button variant="danger" type="submit" id="submitBtn" disabled>
                Submit
              </Button>
            )}
          </Form>
        </Col>
      </Container>
    )
  );
}
